local MogIt,mog = ...;
_G["MogIt"] = mog;

mog.LBB = LibStub("LibBabble-Boss-3.0"):GetLookupTable();
mog.LDB = LibStub("LibDataBroker-1.1");
mog.LDBI = LibStub("LibDBIcon-1.0");
mog.mmb = mog.LDB:NewDataObject(MogIt,{
	type = "launcher",
	icon = "Interface\\Icons\\INV_Enchant_EssenceCosmicGreater",
	OnClick = function(self,btn)
		if mog.frame:IsShown() then
			HideUIPanel(mog.frame);
		else
			ShowUIPanel(mog.frame);
		end
	end,
	OnTooltipShow = function(self)
		if not self or not self.AddLine then return end
		self:AddLine("MogIt");
		self:AddLine("Click to open MogIt.",1,1,1);
	end
});

mog.db = {};
mog.filters = {
	display = {},
	lvl = {},
	faction = {},
	class = {},
	slot = {},
	
	source = {},
	sourceid = {},
	sourceinfo = {},
	zone = {},
	
	colours = {
		[1] = {},
		[2] = {},
		[3] = {},
	},
};
mog.bosses = {};

function mog.register(addon,name,label)
	if not mog.db[addon] then
		mog.db[addon] = {name = name,slots = {}};
	end
	local items = {};
	table.insert(mog.db[addon].slots,{label = label,items = items});
	return items;
end

function mog.addItem(tbl,id,display,lvl,faction,class,slot,source,sourceid,zone,sourceinfo)
	table.insert(tbl,id);
	mog.filters.display[id] = display;
	mog.filters.lvl[id] = lvl;
	mog.filters.faction[id] = faction;
	mog.filters.class[id] = class;
	mog.filters.slot[id] = slot;
	mog.filters.source[id] = source;
	mog.filters.sourceid[id] = sourceid;
	mog.filters.sourceinfo[id] = sourceinfo;
	mog.filters.zone[id] = zone;
end

function mog.addBoss(id,name)
	if not mog.bosses[id] and mog.LBB[name] then
		mog.bosses[id] = mog.LBB[name];
	end
end

function mog.addColours(id,c1,c2,c3)
	if c1 and (not mog.filters.colours[1][id]) then
		mog.filters.colours[1][id] = c1;
		mog.filters.colours[2][id] = c2;
		mog.filters.colours[3][id] = c3;
	end
end

mog.frame = CreateFrame("Frame","MogItFrame",UIParent,"ButtonFrameTemplate");
mog.frame:SetScript("OnEvent",function(self,event,arg1,...)
	if event == "PLAYER_LOGIN" then
		MogIt_Settings = MogIt_Settings or {
			tt = true,
			mouse = true,
			ttDress = false,
			guiDress = false,
			rotate = true,
			mog = true,
		};
		mog.settings = MogIt_Settings;
		if not mog.settings.version then
			DEFAULT_CHAT_FRAME:AddMessage("MogIt has loaded! Type \"/mog\" to open it.");
		end
		mog.settings.minimap = mog.settings.minimap or {};
		mog.settings.url = mog.settings.url or "Battle.net";
		
		mog.settings.ttWidth = mog.settings.ttWidth or 300;
		mog.settings.ttHeight = mog.settings.ttHeight or 300;
		mog.settings.guiWidth = mog.settings.guiWidth or 200;
		mog.settings.guiHeight = mog.settings.guiHeight or 200;
		mog.settings.rows = mog.settings.rows or 2;
		mog.settings.columns = mog.settings.columns or 3;

		mog.settings.posX = mog.settings.posX or 0;
		mog.settings.posY = mog.settings.posY or 0;
		mog.settings.posZ = mog.settings.posZ or 0;
		mog.settings.face = mog.settings.face or 0;
		
		MogIt_Wishlist = MogIt_Wishlist or {};
		mog.wishlist = MogIt_Wishlist;
		mog.wishlist.display = mog.wishlist.display or {};
		mog.wishlist.time = mog.wishlist.time or {};
		
		mog.updateGUI();
		mog.opt.buttonUpdate(true,true);
		mog.manip.model:SetUnit("PLAYER");
		mog.manip.model:SetPosition(0,0,0);
		mog.manip.mass(true);
		
		mog.tooltip.model:SetUnit("PLAYER");
		if mog.settings.rotate then
			mog.tooltip.rotate:Show();
		end
		mog.tooltip:SetSize(mog.settings.ttWidth,mog.settings.ttHeight);
		
		for i=1,GetNumAddOns() do
			local name,_,_,_,loadable = GetAddOnInfo(i);
			name = name:match("^(MogIt_.+)");
			if name and loadable then
				mog.db[name] = mog.db[name] or false;
			end
		end
		
		mog.settings.version = GetAddOnMetadata(MogIt,"Version");
		mog.wishlist.version = GetAddOnMetadata(MogIt,"Version");
		
		mog.LDBI:Register(MogIt,mog.mmb,mog.settings.minimap);
		mog.frame:UnregisterEvent("PLAYER_LOGIN");
	elseif event == "GET_ITEM_INFO_RECEIVED" then
		if mog.over and mog.over == GameTooltip:GetOwner() then
			mog.updateTooltip(mog.over);
		end
	elseif event == "ADDON_LOADED" then
		if arg1 == "AtlasLoot" then
			if AtlasLootTooltipTEMP then
				AtlasLootTooltipTEMP:HookScript("OnTooltipSetItem",mog.tooltip.ShowItem);
				AtlasLootTooltipTEMP:HookScript("OnHide",mog.tooltip.HideItem);
			end
		else
			local addon = arg1:match("^(MogIt_.+)");
			if addon then
				if mog.frame:IsShown() then
					ToggleDropDownMenu(1,addon,mog.dd);
				end
			end
		end
	end
end);
mog.frame:RegisterEvent("PLAYER_LOGIN");
mog.frame:RegisterEvent("GET_ITEM_INFO_RECEIVED");
mog.frame:RegisterEvent("ADDON_LOADED");

SLASH_MOGIT1 = "/mog";
SLASH_MOGIT2 = "/mogit";
SlashCmdList["MOGIT"] = function()
	if mog.frame:IsShown() then
		HideUIPanel(mog.frame);
	else
		ShowUIPanel(mog.frame);
	end
end

mog.interface = CreateFrame("Frame");
mog.interface.title = mog.interface:CreateFontString(nil,"ARTWORK","GameFontNormalLarge");
mog.interface.title:SetPoint("TOPLEFT",16,-16 );
mog.interface.title:SetText("MogIt");
mog.interface.launch = CreateFrame("Button",nil,mog.interface,"UIPanelButtonTemplate");
mog.interface.launch:SetWidth(120);
mog.interface.launch:SetHeight(22);
mog.interface.launch:SetPoint("TOPLEFT",mog.interface.title,"BOTTOMLEFT",0,-8);
mog.interface.launch:SetText("Launch MogIt");
mog.interface.launch:SetScript("OnClick",function(self,btn)
	InterfaceOptionsFrame.lastFrame = nil;
	HideUIPanel(InterfaceOptionsFrame);
	ShowUIPanel(mog.frame);
end);
mog.interface.name = "MogIt";
InterfaceOptions_AddCategory(mog.interface);

table.insert(UISpecialFrames,"MogItFrame");
mog.frame:Hide();
mog.frame:SetPoint("CENTER",UIParent,"CENTER");
mog.frame:SetSize(220,342);
mog.frame:SetFrameStrata("DIALOG");
mog.frame:SetClampedToScreen(true);
mog.frame:EnableMouse(true);
mog.frame:SetMovable(true);
mog.frame:SetUserPlaced(true);
mog.frame:SetScript("OnMouseDown",function(self) 
	self:StartMoving();
end);
mog.frame:SetScript("OnMouseUp",function(self) 
	self:StopMovingOrSizing();
end);

mog.frame.TitleText:SetText(MogIt);
mog.frame.TitleText:SetPoint("RIGHT",mog.frame,"RIGHT",-28,0);

mog.frame.portrait:SetTexture("Interface\\AddOns\\MogIt\\Images\\MogIt");
mog.frame.portrait:SetTexCoord(0,106/128,0,105/128);

mog.frame.expand = CreateFrame("Button",nil,mog.frame);
mog.frame.expand:SetWidth(27);
mog.frame.expand:SetHeight(27);
mog.frame.expand:SetPoint("TOPRIGHT",mog.frame.Inset,"BOTTOMRIGHT",2,2);
mog.frame.expand:SetScript("OnClick",function(self)
	if mog.grid:IsShown() then
		mog.grid:Hide();
	else
		mog.grid:Show();
	end
end);
mog.frame.expand:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
mog.frame.expand:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");
mog.frame.expand:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled");
mog.frame.expand:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight","ADD");

mog.frame.tab1 = CreateFrame("Button","MogItFrameTab1",mog.frame,"CharacterFrameTabButtonTemplate");
mog.frame.tab1:SetPoint("TOPLEFT",mog.frame,"BOTTOMLEFT",1,2);
mog.frame.tab1:SetText("Controls");
mog.frame.tab1:SetID(1);
mog.frame.tab1:SetScript("OnLoad",nil);
mog.frame.tab1:SetScript("OnEvent",nil);
mog.frame.tab1:SetScript("OnEnter",nil);
mog.frame.tab1:SetScript("OnLeave",nil);
mog.frame.tab1:SetScript("OnShow",function(self)
	PanelTemplates_TabResize(self, 0, nil, 36, 88);
end);
mog.frame.tab1:SetScript("OnClick",function(self)
	PanelTemplates_Tab_OnClick(self, mog.frame);
	mog.manip:Show();
	mog.filt:Hide();
	mog.optscroll:Hide();
end);

mog.frame.tab2 = CreateFrame("Button","MogItFrameTab2",mog.frame,"CharacterFrameTabButtonTemplate");
mog.frame.tab2:SetPoint("LEFT",mog.frame.tab1,"RIGHT",-15,0);
mog.frame.tab2:SetText("Filters");
mog.frame.tab2:SetID(2);
mog.frame.tab2:SetScript("OnLoad",nil);
mog.frame.tab2:SetScript("OnEvent",nil);
mog.frame.tab2:SetScript("OnEnter",nil);
mog.frame.tab2:SetScript("OnLeave",nil);
mog.frame.tab2:SetScript("OnShow",function(self)
	PanelTemplates_TabResize(self, 0, nil, 36, 88);
end);
mog.frame.tab2:SetScript("OnClick",function(self)
	PanelTemplates_Tab_OnClick(self, mog.frame);
	mog.manip:Hide();
	mog.filt:Show();
	mog.optscroll:Hide();
end);

mog.frame.tab3 = CreateFrame("Button","MogItFrameTab3",mog.frame,"CharacterFrameTabButtonTemplate");
mog.frame.tab3:SetPoint("LEFT",mog.frame.tab2,"RIGHT",-15,0);
mog.frame.tab3:SetText("Options");
mog.frame.tab3:SetID(3);
mog.frame.tab3:SetScript("OnLoad",nil);
mog.frame.tab3:SetScript("OnEvent",nil);
mog.frame.tab3:SetScript("OnEnter",nil);
mog.frame.tab3:SetScript("OnLeave",nil);
mog.frame.tab3:SetScript("OnShow",function(self)
	PanelTemplates_TabResize(self, 0, nil, 36, 88);
end);
mog.frame.tab3:SetScript("OnClick",function(self)
	PanelTemplates_Tab_OnClick(self, mog.frame);
	mog.manip:Hide();
	mog.filt:Hide();
	mog.optscroll:Show();
end);

PanelTemplates_SetNumTabs(mog.frame,3);
PanelTemplates_SetTab(mog.frame,1);