local MogIt,mog = ...;

local IsShiftKeyDown = IsShiftKeyDown;
local IsControlKeyDown = IsControlKeyDown;
local GetScreenWidth = GetScreenWidth;
local GetScreenHeight = GetScreenHeight;
local GetMouseFocus = GetMouseFocus;
local GetItemInfo = GetItemInfo;
local GetSpellInfo = GetSpellInfo;
local GetAchievementInfo = GetAchievementInfo;
local type = type;
local pairs = pairs;
local ipairs = ipairs;
local tinsert = table.insert;
local tremove = table.remove;
local floor = math.floor;
local ceil = math.ceil;
local band = bit.band;
local min = min;
local tonumber = tonumber;

mog.models = {};
mog.bin = {};
mog.list = {};
mog.display = {};
mog.source = {
	[1] = "Drop",
	[2] = "PvP",
	[3] = "Quest",
	[4] = "Vendor",
	[5] = "Crafted",
	[6] = "Achievement",
	[7] = "Code Redemption",
};
mog.diffs = {
	--[1] = "5N",
	[2] = "Heroic",
	[3] = "10N",
	[4] = "25N",
	[5] = "10H",
	[6] = "25H",
	[7] = "Heroic",
};
mog.slots = {
	[1] = "One Hand";
	[2] = "Main Hand";
	[3] = "Off Hand";
};
mog.professions = {
	[1] = GetSpellInfo(2259), -- Alchemy
	[2] = GetSpellInfo(2018), -- Blacksmithing
	[3] = GetSpellInfo(7411), -- Enchanting
	[4] = GetSpellInfo(4036), -- Engineering
	[5] = GetSpellInfo(45357), -- Inscription
	[6] = GetSpellInfo(25229), -- Jewelcrafting
	[7] = GetSpellInfo(2108), -- Leatherworking
	[8] = GetSpellInfo(3908), -- Tailoring

	[9] = GetSpellInfo(2366), -- Herbalism
	[10] = GetSpellInfo(2575), -- Mining
	[11] = GetSpellInfo(8613), -- Skinning

	[12] = GetSpellInfo(78670), -- Archaeology
	[13] = GetSpellInfo(2550), -- Cooking
	[14] = GetSpellInfo(3273), -- First Aid
	[15] = GetSpellInfo(7620), -- Fishing
};
mog.urlList = {
	["Wowhead"] = {url="http://www.wowhead.com/item=",fav="fav_wh"},
	["MMO-Champion"] = {url="http://db.mmo-champion.com/i/",fav="fav_mmo"},
	["Battle.net"] = {url="http://eu.battle.net/wow/en/item/",fav="fav_wow"},
};

mog.grid = MogItGrid;
mog.grid:Hide();
mog.grid:SetPoint("TOPLEFT",mog.frame,"TOPRIGHT",10,-10);
mog.grid:EnableMouse(true);
mog.grid:EnableMouseWheel(true);
mog.grid:SetResizable(true);
mog.grid:SetClampedToScreen(true);
mog.grid:SetFrameStrata("HIGH");
mog.grid:SetHitRectInsets(-10,-10,-10,-10);
mog.grid:SetScript("OnShow",function(self)
	mog.scroll:update();
	mog.frame.expand:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
	mog.frame.expand:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
	mog.frame.expand:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled");
end);
mog.grid:SetScript("OnHide",function(self)
	mog.frame.expand:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
	mog.frame.expand:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");
	mog.frame.expand:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled");
end);

mog.grid.resize = CreateFrame("Frame",nil,mog.grid);
mog.grid.resize:SetSize(28,28);
mog.grid.resize:SetPoint("BOTTOMRIGHT",mog.grid,"BOTTOMRIGHT",10,-10);
mog.grid.resize:EnableMouse(true);
mog.grid.resize:SetScript("OnMouseDown",function(self)
	mog.grid:SetMinResize((mog.settings.columns*35)-5+20+43,(mog.settings.rows*35)-5+43+20);
	mog.grid:SetMaxResize(GetScreenWidth(),GetScreenHeight());
	mog.grid:StartSizing();
	mog.grid.resize.update:Show();
end);
mog.grid.resize:SetScript("OnMouseUp",function(self)
	mog.grid.resize.update:Hide();
	mog.grid:StopMovingOrSizing();
	mog.grid:ClearAllPoints();
	mog.grid:SetPoint("TOPLEFT",mog.frame,"TOPRIGHT",10,-10);
	mog.opt.buttonUpdate(true,true);
end);
mog.grid.resize:SetScript("OnHide",function(self)
	self:GetScript("OnMouseUp")(self);
end);
mog.grid.resize.update = CreateFrame("Frame",nil,mog.grid.resize);
mog.grid.resize.update:Hide();
mog.grid.resize.update:SetScript("OnUpdate",function(self)
	mog.settings.guiWidth = floor((mog.grid:GetWidth()+5-20-43)/mog.settings.columns)-5;
	mog.settings.guiHeight = floor((mog.grid:GetHeight()+5-43-20)/mog.settings.rows)-5;
	mog.grid:ClearAllPoints();
	mog.grid:SetPoint("TOPLEFT",mog.frame,"TOPRIGHT",10,-10);
	mog.updateGUI(true);
end);
mog.grid.resize.texture = mog.grid.resize:CreateTexture(nil,"OVERLAY");
mog.grid.resize.texture:SetSize(16,16);
mog.grid.resize.texture:SetTexture("Interface\\AddOns\\MogIt\\Images\\Resize");
mog.grid.resize.texture:SetPoint("BOTTOMRIGHT",mog.grid.resize,"BOTTOMRIGHT",-3,3);

mog.grid.topbar = CreateFrame("Frame",nil,mog.grid);
mog.grid.topbar:SetHeight(18);
mog.grid.topbar:SetPoint("TOPLEFT",mog.grid,"TOPLEFT",20,-20);
mog.grid.topbar:SetPoint("TOPRIGHT",mog.grid,"TOPRIGHT",-20,-20);
mog.grid.topbar:SetBackdrop({
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	--edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, --edgeSize = 16, 
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
});
mog.grid.topbar:SetBackdropColor(0,0,0,0.6);

mog.grid.topleft = mog.grid.topbar:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
mog.grid.topleft:SetPoint("LEFT",mog.grid.topbar,"LEFT",5,0);

mog.grid.topright = mog.grid.topbar:CreateFontString(nil,"ARTWORK","GameFontHighlightSmall");
mog.grid.topright:SetPoint("RIGHT",mog.grid.topbar,"RIGHT",-5,0);

mog.scroll = CreateFrame("Slider","MogItScroll",mog.grid,"UIPanelScrollBarTemplate");
mog.scroll:SetBackdrop({
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	--edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, --edgeSize = 16, 
	insets = { left = 0, right = 0, top = 0, bottom = 0 }
});
mog.scroll:SetBackdropColor(0,0,0,0.6);
mog.scroll:SetScript("OnValueChanged",function(self,value)
	mog.scroll:update(value);
end);
mog.scroll:SetValueStep(1);
mog.scroll:SetValue(1);
mog.scroll.up = MogItScrollScrollUpButton;
mog.scroll.down = MogItScrollScrollDownButton;
mog.scroll:SetPoint("TOPRIGHT",mog.grid.topbar,"BOTTOMRIGHT",0,-21);
mog.scroll:SetPoint("BOTTOMRIGHT",mog.grid,"BOTTOMRIGHT",-20,36);
mog.scroll.up:SetScript("OnClick",function(self)
	mog.scroll:update(nil,-1);
end);
mog.scroll.down:SetScript("OnClick",function(self)
	mog.scroll:update(nil,1);
end);

function mog.addModel()
	local f;
	if mog.bin[1] then
		f = mog.bin[1];
		tremove(mog.bin,1);
	else
		f = CreateFrame("DressUpModel",nil,mog.grid);
		f:Hide();
		f:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
			--edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true, tileSize = 16, --edgeSize = 16, 
			insets = { left = 0, right = 0, top = 0, bottom = 0 }
		});
		f:SetBackdropColor(0,0,0,0.6);
		f:SetUnit("PLAYER");
		f:SetModelScale(2);
		f:SetPosition(0,0,0);
		f:EnableMouse(true);
		f:SetScript("OnMouseUp",function(self,btn)
			if GetMouseFocus() == self then
				if btn == "LeftButton" then
					if IsShiftKeyDown() then
						local _,link = GetItemInfo(self.item);
						if link then
							ChatEdit_InsertLink(link);
						end
					elseif IsControlKeyDown() then
						DressUpItemLink(self.item);
					else
						if type(self.list) == "table" then
							self.cycle = (self.cycle < #self.list and (self.cycle + 1)) or 1;
							self.item = self.list[self.cycle];
							mog.updateTooltip(self);
						end
					end
				elseif btn == "RightButton" then
					if IsControlKeyDown() then
						mog.manip.model:TryOn(self.item);
					elseif IsShiftKeyDown() then
						StaticPopup_Show("MOGIT_ITEMURL",mog.settings.url,nil,self.item);
					else
						if mog.wishlist.display[self.display] then
							mog.wishlist.display[self.display] = nil;
							mog.wishlist.time[self.display] = nil;
							if mog.selected == "wl" then
								table.remove(mog.list,self.index);
								mog.display[self.display] = nil;
								mog.scroll:update();
								mog.grid.topleft:SetText(#mog.list.." models");
							else
								self.star:Hide();
							end
						else
							mog.wishlist.display[self.display] = self.list;
							mog.wishlist.time[self.display] = time();
							if mog.selected == "wl" then
								mog.scroll:update();
								mog.grid.topleft:SetText(#mog.list.." models");
							else
								self.star:Show();
							end
						end
					end
				end
			end
		end);
		f:SetScript("OnEnter",mog.updateTooltip);
		f:SetScript("OnLeave",function(self)
			mog.over = nil;
			if GameTooltip:GetOwner() == self then
				GameTooltip:Hide();
			end
		end);
		f:SetScript("OnHide",function(self)
			self:SetPosition(0,0,0);
		end);
		f:SetScript("OnShow",function(self)
			self:SetPosition(mog.settings.posZ,mog.settings.posX,mog.settings.posY);
		end);
		f.star = f:CreateTexture(nil,"ARTWORK");
		f.star:SetSize(16,16);
		f.star:SetPoint("TOPRIGHT",f,"TOPRIGHT",-4,-4);
		f.star:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_1");
		f.star:Hide();
		
		--[[f.c = {};
		f.c[1] = f:CreateTexture(nil,"OVERLAY");
		f.c[1]:SetSize(16,16);
		f.c[1]:SetPoint("BOTTOMLEFT",f,"BOTTOMLEFT",4,4);
		f.c[2] = f:CreateTexture(nil,"OVERLAY");
		f.c[2]:SetSize(16,16);
		f.c[2]:SetPoint("LEFT",f.c[1],"RIGHT",2,0);
		f.c[3] = f:CreateTexture(nil,"OVERLAY");
		f.c[3]:SetSize(16,16);
		f.c[3]:SetPoint("LEFT",f.c[2],"RIGHT",2,0);--]]
	end
	tinsert(mog.models,f);
end

function mog.delModel(f)
	mog.models[f]:Hide();
	tinsert(mog.bin,mog.models[f]);
	tremove(mog.models,f);
end

function mog.updateGUI(resize)
	local rows,columns = mog.settings.rows,mog.settings.columns;
	local total = rows*columns;
	local current = #mog.models;
	local width,height = mog.settings.guiWidth,mog.settings.guiHeight;
	
	if not resize then
		if current > total then
			for i=current,(total+1),-1 do
				mog.delModel(i);
			end
		elseif current < total then
			for i=(current+1),total,1 do
				mog.addModel();
			end
		end
		
		mog.grid:SetSize(((width+5)*columns)-5+20+43,((height+5)*rows)-5+43+20);
	end
	
	for row=1,rows do
		for column=1,columns do
			local n = ((row-1)*columns)+column;
			if n==1 then
				mog.models[n]:SetPoint("TOPLEFT",mog.grid.topbar,"BOTTOMLEFT",0,-5);
			elseif column==1 then
				mog.models[n]:SetPoint("TOPLEFT",mog.models[n-columns],"BOTTOMLEFT",0,-5);
			else
				mog.models[n]:SetPoint("TOPLEFT",mog.models[n-1],"TOPRIGHT",5,0);
			end
			mog.models[n]:SetSize(width,height);
		end
	end
end

function mog.scroll.update(self,page,offset)
	local models = #mog.models;
	local total = ceil(#mog.list/models);
	
	if total > 1 then
		self:SetMinMaxValues(1,total);
		self:Show();
	else
		self:Hide();
	end
	
	if not page then
		page = self:GetValue() or 1;
	end
	
	if offset then
		page = page + offset;
	end
	
	if page < 1 then
		page = 1;
	elseif page > total then
		page = total;
	end
	self:SetValue(page);
	
	if page == 1 then
		self.up:Disable();
	else
		self.up:Enable();
	end
	if page == total then
		self.down:Disable();
	else
		self.down:Enable();
	end
	
	local id,model,index;
	for id,model in ipairs(mog.models) do
		index = ((page-1)*models)+id;
		if mog.list[index] then
			model.display = mog.list[index];
			model.list = mog.display[model.display];
			model.cycle = 1;
			model.item = type(model.list) == "table" and model.list[1] or model.list;
			model.index = index;
			model:Show();
			if mog.settings.guiDress then
				model:Dress();
			else
				model:Undress();
			end
			model:TryOn(model.item);
			if mog.wishlist.display[model.display] then
				model.star:Show();
			else
				model.star:Hide();
			end
			
			--[[for i=1,3 do
				if mog.filters.colours[i][model.display] then
					model.c[i]:Show();
					local r,g,b = mog.filters.colours[i][model.display]:match("^(..)(..)(..)$");
					r = tonumber(r,16);
					g = tonumber(g,16);
					b = tonumber(b,16); 
					model.c[i]:SetTexture(r/255,g/255,b/255,1);
				else
					model.c[i]:Hide();
				end
			end--]]
		else
			model:Hide();
		end
	end
	if mog.over and mog.over == GameTooltip:GetOwner() then
		mog.updateTooltip(mog.over);
	end
	
	if total > 0 then
		mog.grid.topright:Show();
		mog.grid.topright:SetText(MERCHANT_PAGE_NUMBER:format(page,total));
	else
		mog.grid.topright:Hide();
	end
end

mog.grid:SetScript("OnMouseWheel",function(self,offset)
	local value = mog.scroll:GetValue();
	local low,high = mog.scroll:GetMinMaxValues();
	if (offset > 0 and value > low) then
		mog.scroll:update(nil,-1);
	elseif (offset < 0 and value < high) then
		mog.scroll:update(nil,1);
	end
end);

function mog.buildList()
	if not mog.selected then return end;
	wipe(mog.list);
	wipe(mog.display);
	if mog.selected == "wl" then
		for k,v in pairs(mog.wishlist.display) do
			table.insert(mog.list,k);
			mog.display[k] = v;
		end
		table.sort(mog.list,function(a,b)
			if mog.wishlist.time[a] == mog.wishlist.time[b] then
				return a > b;
			else
				return mog.wishlist.time[a] > mog.wishlist.time[b];
			end
		end);
	else
		for k,v in ipairs(mog.selected.items) do
			if mog.filterLevel(v) and mog.filterFaction(v) and mog.filterClass(v) and mog.filterSlot(v) and mog.filterSource(v) then
				local disp = mog.filters.display[v];
				if not mog.display[disp] then
					mog.display[disp] = v;
					tinsert(mog.list,disp);
				elseif type(mog.display[disp]) == "table" then
					tinsert(mog.display[disp],v);
				else
					mog.display[disp] = {mog.display[disp],v};
				end
			end
		end
		mog.sort();
	end
	mog.scroll:update(1);
	mog.grid:Show();
	mog.grid.topleft:SetText(#mog.list.." models");
end

function mog.filterLevel(v)
	return ((mog.filters.lvl[v] or 0) >= mog.filt._minlvl) and ((mog.filters.lvl[v] or 0) <= mog.filt._maxlvl);
end

function mog.filterFaction(v)
	return (not mog.filters.faction[v]) or (mog.filters.faction[v] == 1 and mog.filt._alliance) or (mog.filters.faction[v] == 2 and mog.filt._horde);
end

function mog.filterClass(v)
	return (not mog.filters.class[v]) or (band(mog.filters.class[v],mog.filt._class) > 0); 
end

function mog.filterSlot(v)
	return (not mog.filters.slot[v]) or mog.filt._slots[mog.filters.slot[v]];
end

function mog.filterSource(v)
	return (not mog.filters.source[v]) or mog.filt._sources[mog.filters.source[v]];
end

local colourCache = {};
local itemCache = {};
function mog.sort()
	wipe(colourCache);
	wipe(itemCache);
	if mog.sorting == "level" then
		table.sort(mog.list,function(a,b)
			local aI,bI = mog.minItem(a),mog.minItem(b);
			if mog.filters.lvl[aI] == mog.filters.lvl[bI] then
				return aI > bI;
			else
				return (mog.filters.lvl[aI] or 0) > (mog.filters.lvl[bI] or 0);
			end
		end);
	elseif mog.sorting == "colour" then
		table.sort(mog.list,function(a,b)
			local aS,bS = mog.colourScore(a),mog.colourScore(b);
			if aS == bS then
				return mog.minItem(a) > mog.minItem(b);
			else
				return aS < bS;
			end
		end);
	end
end

function mog.colourScore(display)
	if not colourCache[display] then
		--local d1,d2;
		local distance = 195075;
		for i=1,3 do
			if mog.filters.colours[i][display] then
				local r,g,b = mog.filters.colours[i][display]:match("^(..)(..)(..)$");
				r = tonumber(r,16);
				g = tonumber(g,16);
				b = tonumber(b,16);
				local dist = ((mog.cR-r)^2)+((mog.cG-g)^2)+((mog.cB-b)^2);
				if dist < distance then
					distance = dist;
				end
				--[[if (not d1) or dist < d1 then
					d2 = d1 or dist;
					d1 = dist;
				elseif dist < d2 then
					d2 = dist;
				end--]]
			else
				break;
			end
		end
		--local distance = (195075-(d1 or 195075))^2+(195075-(d2 or 195075))^2;
		colourCache[display] = distance;
	end
	return colourCache[display];
end

function mog.minItem(display)
	if not itemCache[display] then
		if type(mog.display[display]) == "table" then
			for k,v in ipairs(mog.display[display]) do
				if (not itemCache[display]) or (not mog.filters.lvl[itemCache[display]]) or (mog.filters.lvl[v] and (mog.filters.lvl[v] < mog.filters.lvl[itemCache[display]])) then
					itemCache[display] = v;
				end
			end
		else
			itemCache[display] = mog.display[display];
		end
	end
	return itemCache[display];
end

StaticPopupDialogs["MOGIT_ITEMURL"] = {
	text = "%s Item URL",
	button1 = CLOSE,
	hasEditBox = 1,
	maxLetters = 255,
	editBoxWidth = 260,
	OnShow = function(self,item)
		self.editBox:SetText(mog.urlList[mog.settings.url].url..item);
		self.editBox:SetFocus();
		self.editBox:HighlightText();
	end,
	EditBoxOnEnterPressed = function(self)
		self:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
};

function mog.updateTooltip(self)
	mog.over = self;
	GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
	--GameTooltip:ClearLines();
	local name,link = GetItemInfo(self.item);
	--GameTooltip:AddLine(self.display,1,1,1);
	--GameTooltip:AddLine(" ");
	GameTooltip:AddDoubleLine(link or ("["..(name or "Unknown").."]"),((type(self.list) == "table") and ("Item "..self.cycle.."/"..#self.list)),1,0,0,1,0,0);
	if mog.filters.source[self.item] then
		GameTooltip:AddDoubleLine("Source:",mog.source[mog.filters.source[self.item]],nil,nil,nil,1,1,1);
		if mog.filters.sourceid[self.item] then
			if mog.filters.source[self.item] == 1 then -- Drop
				if mog.bosses[mog.filters.sourceid[self.item]] then
					GameTooltip:AddDoubleLine("Boss:",mog.bosses[mog.filters.sourceid[self.item]],nil,nil,nil,1,1,1);
				end
			--elseif mog.filters.source[self.item] == 3 then -- Quest
			elseif mog.filters.source[self.item] == 5 then -- Crafted
				if mog.filters.sourceinfo[self.item] then
					GameTooltip:AddDoubleLine("Profession:",mog.professions[mog.filters.sourceinfo[self.item]],nil,nil,nil,1,1,1);
				end
			elseif mog.filters.source[self.item] == 6 then -- Achievement
				if mog.filters.sourceid[self.item] then
					local _,name = GetAchievementInfo(mog.filters.sourceid[self.item]);
					GameTooltip:AddDoubleLine("Achievement:",name,nil,nil,nil,1,1,1);
				end
			end
		end
	end
	if mog.filters.zone[self.item] then
		local zone = GetMapNameByID(mog.filters.zone[self.item]);
		if mog.filters.source[self.item] == 1 and mog.diffs[mog.filters.sourceinfo[self.item]] then
			zone = zone.." ("..mog.diffs[mog.filters.sourceinfo[self.item]]..")";
		end
		GameTooltip:AddDoubleLine("Zone:",zone,nil,nil,nil,1,1,1);
	end
	
	GameTooltip:AddLine(" ");
	GameTooltip:AddDoubleLine("ID:",self.item,nil,nil,nil,1,1,1);
	if mog.filters.lvl[self.item] then
		GameTooltip:AddDoubleLine("Level:",mog.filters.lvl[self.item],nil,nil,nil,1,1,1);
	end
	if mog.filters.faction[self.item] then
		GameTooltip:AddDoubleLine("Faction:",(mog.filters.faction[self.item] == 1 and "Alliance" or "Horde"),nil,nil,nil,1,1,1);
	end
	if mog.filters.class[self.item] and mog.filters.class[self.item] > 0 then
		local str;
		for k,v in pairs(mog.classBits) do
			if band(mog.filters.class[self.item],v) > 0 then
				if str then
					str = str..", "..string.format("\124cff%.2x%.2x%.2x",RAID_CLASS_COLORS[k].r*255,RAID_CLASS_COLORS[k].g*255,RAID_CLASS_COLORS[k].b*255)..mog.classes[k].."\124r";
				else
					str = string.format("\124cff%.2x%.2x%.2x",RAID_CLASS_COLORS[k].r*255,RAID_CLASS_COLORS[k].g*255,RAID_CLASS_COLORS[k].b*255)..mog.classes[k].."\124r";
				end
			end
		end
		GameTooltip:AddDoubleLine("Class:",str,nil,nil,nil,1,1,1);
	end
	if mog.filters.slot[self.item] then
		GameTooltip:AddDoubleLine("Slot:",mog.slots[mog.filters.slot[self.item]],nil,nil,nil,1,1,1);
	end
	
	GameTooltip:AddLine(" ");
	GameTooltip:AddLine("Controls");
	GameTooltip:AddDoubleLine("Scroll through list","Scroll wheel",0,1,0,1,1,1);
	GameTooltip:AddDoubleLine("Change item","Left click",0,1,0,1,1,1);
	GameTooltip:AddDoubleLine("Chat link","Shift + Left click",0,1,0,1,1,1);
	GameTooltip:AddDoubleLine("Try on","Ctrl + Left click",0,1,0,1,1,1);
	GameTooltip:AddDoubleLine((mog.wishlist.display[self.display] and "Delete from" or "Add to").." wishlist","Right click",0,1,0,1,1,1);
	GameTooltip:AddDoubleLine("Item URL","Shift + Right click",0,1,0,1,1,1);
	GameTooltip:AddDoubleLine("Add to control model","Ctrl + Right click",0,1,0,1,1,1);
	GameTooltip:Show();
end

mog.dd = CreateFrame("Frame","MogItDropdown",mog.frame,"UIDropDownMenuTemplate");
mog.dd:SetPoint("BOTTOMRIGHT",mog.frame.Inset,"TOPRIGHT",15,0);
UIDropDownMenu_SetWidth(mog.dd,140);
UIDropDownMenu_SetButtonWidth(mog.dd,155);
UIDropDownMenu_JustifyText(mog.dd,"LEFT");
UIDropDownMenu_SetText(mog.dd,"Select a category");
function mog.dd:initialize(tier)
	local info;
	if tier == 1 then
		info = UIDropDownMenu_CreateInfo();
		info.text = "\124cFFFFFF00Wishlist\124r";
		info.value = "wl";
		info.notCheckable = true;
		info.func = function(self)
			UIDropDownMenu_SetText(mog.dd,"Wishlist");
			mog.selected = self.value;
			mog.buildList();
		end
		UIDropDownMenu_AddButton(info,tier);
		for k,v in pairs(mog.db) do
			info = UIDropDownMenu_CreateInfo();
			info.text = "\124cFF"..(v and "00FF00" or "FF0000")..(v and v.name or k:match("^MogIt_(.+)")).."\124r"..(v and "" or " (Click to load addon)");
			info.value = k;
			info.hasArrow = v and true;
			info.keepShownOnClick = v and true;
			info.notCheckable = true;
			info.func = function(self)
				if not mog.db[self.value] then
					LoadAddOn(self.value);
				end
			end
			UIDropDownMenu_AddButton(info,tier);
		end
	elseif tier == 2 then
		for k,v in ipairs(mog.db[UIDROPDOWNMENU_MENU_VALUE].slots) do
			info = UIDropDownMenu_CreateInfo();
			info.text = v.label;
			info.value = v;
			info.notCheckable = true;
			info.func = function(self)
				UIDropDownMenu_SetText(mog.dd,mog.db[self.arg1].name.." - "..self.value.label);
				mog.selected = self.value;
				mog.buildList();
				CloseDropDownMenus();
			end
			info.arg1 = UIDROPDOWNMENU_MENU_VALUE;
			UIDropDownMenu_AddButton(info,tier);
		end
	end
end